import json
import datetime
import time
import smtplib
import ssl
import hashlib
import secrets
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.utils import formataddr
import requests
from flask import *
from flaskext.mysql import MySQL
from pymysql.cursors import DictCursor

app = Flask(__name__)
config = json.load(open("config.json"))
app.config["SECRET_KEY"] = config["SECRET_KEY"]
app.config["MYSQL_DATABASE_USER"] = config["db"]["user"]
app.config["MYSQL_DATABASE_PASSWORD"] = config["db"]["password"]
app.config["MYSQL_DATABASE_DB"] = config["db"]["db_name"]

mysql = MySQL(cursorclass=DictCursor)
mysql.init_app(app)


def encrypt_password(password):
    return hashlib.sha512((password + password[::-1]).encode()).hexdigest()


def send_email(email, subject, message):
    context = ssl.create_default_context()
    with smtplib.SMTP_SSL(config["smtp"]["host"], config["smtp"]["port"], context=context) as server:
        server.login(config["smtp"]["sender"], config["smtp"]["password"])
        msg = MIMEMultipart("alternative")
        msg["Subject"] = subject
        msg["From"] = formataddr(
            (config["app_name"], config["smtp"]["sender"]))
        msg["To"] = email
        msg.attach(MIMEText(message, "plain"))
        server.sendmail(config["smtp"]["sender"], email, msg.as_string())


@app.before_request
def before_request_callback():
    session["app_name"] = config["app_name"]
    session["support_mail"] = config["support_mail"]
    session["support_phone"] = config["support_phone"]
    session["support_address"] = config["support_address"]
    session["plans"] = list(config['plans'].values())
    session["coins"] = list(config['address'].keys())


@app.route("/", methods=["GET", "POST"])
def index():
    if request.method == "POST":
        name = request.form.get("name", "").strip()
        email = request.form.get("email", "").strip().lower()
        message = request.form.get("message", "").strip()

        db_connection = mysql.connect()
        cursor = db_connection.cursor()
        cursor.execute(
            "INSERT INTO contact_messages (name, email, message) VALUES (%s, %s, %s)", (name, email, message))
        db_connection.commit()
        return redirect(url_for("index") + "?sent=true")
    return render_template("index.html")


@app.route("/register/", methods=["GET", "POST"])
def register():
    if request.method == "POST":
        fullname = request.form.get("fullname", "").strip()
        username = request.form.get("username", "").strip().lower()
        email = request.form.get("email", "").strip().lower()
        password = request.form.get("password", "").strip()
        phone = request.form.get("phone", "").strip()
        country = request.form.get("country", "").strip().lower()
        repeat_password = request.form.get("repeat_password", "").strip()
        ref = request.args.get("ref")

        db_connection = mysql.connect()
        cursor = db_connection.cursor()
        if password != repeat_password:
            return redirect(url_for("register") + "?error=1")

        passwordRaw = password
        password = encrypt_password(password)

        cursor.execute(
            "SELECT * FROM users WHERE email=%s OR username=%s", (email, username))
        if cursor.fetchone() != None:
            return redirect(url_for("register") + "?error=2")

        cursor.execute("INSERT INTO users (fullname, username, email, password, phone, address, postal, country, dob, ref) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s)",
                       (fullname, username, email, password, phone, "", "", country, "", ref))
        db_connection.commit()

        send_email(email, f"Welcome to {config['app_name']}",
                   f"Hello {fullname}! \n\nYour registration was successful. Here are your account credentials, please keep them safe. \n\nUsername: {username} \n\nEmail Address: {email} \n\nPassword: {passwordRaw} \n\nPlease do not disclose these credentials to anyone to avoid risks and loss of funds due to theft or fraud. \n\nHappy Trading! \n\n{config['app_name']} Team ")

        return redirect(url_for("login") + "?registered=true")
    return render_template("in/register.html")


@app.route("/login/", methods=["GET", "POST"])
def login():
    if "user" in session:
        return redirect(url_for("dashboard"))

    if request.method == "POST":
        email = request.form.get("email", "").strip().lower()
        password = request.form.get("password", "").strip()

        db_connection = mysql.connect()
        cursor = db_connection.cursor()
        cursor.execute(
            "SELECT * FROM users WHERE email=%s AND password=%s", (email, encrypt_password(password)))
        user = cursor.fetchone()
        if user == None:
            return redirect(url_for("login") + "?error=1")
        else:
            session["user"] = {
                "id": user["id"],
                "admin": user["admin"]
            }
            session.permanent = True
            return redirect(url_for("dashboard"))
    return render_template("in/login.html")


@app.route("/reset-password/", methods=["GET", "POST"])
def reset_password():
    if request.method == "POST":
        email = request.form.get("email", "").strip().lower()

        db_connection = mysql.connect()
        cursor = db_connection.cursor()
        cursor.execute("SELECT * FROM users WHERE email=%s", email)
        user = cursor.fetchone()

        if user:
            password = secrets.token_urlsafe(10)
            cursor.execute("UPDATE users SET password=%s WHERE id=%s",
                           (encrypt_password(password), user["id"]))
            db_connection.commit()
            send_email(user["email"], f"New password for your {config['app_name']} account",
                       f"We received a request to reset your password on https://{config['site_url']} and therefore generated a new one for you.\n\nLogin to your account using your email and the temporary password {password}\n\nPlease don't forget to change the temporary password once you login to your account")
            return redirect(url_for("reset_password") + "?reset=true")

    return render_template("in/reset-password.html")


@app.route("/dashboard/")
def dashboard():
    if "user" not in session:
        return redirect(url_for("index"))

    db_connection = mysql.connect()
    cursor = db_connection.cursor()

    cursor.execute("SELECT * FROM users WHERE id=%s", (session["user"]["id"],))
    user = cursor.fetchone()
    cursor.execute(
        "SELECT COUNT(*) from messages WHERE user_id=%s", user["id"])
    user["total_messages"] = cursor.fetchone()["COUNT(*)"]

    if user["admin"] == 2:
        return redirect(url_for("suspended"))

    site_details = {}
    if user["admin"] == 1:
        cursor.execute("SELECT COUNT(*) from users")
        site_details["total_users"] = cursor.fetchone()["COUNT(*)"]
        cursor.execute("SELECT SUM(amount) FROM deposits")
        site_details["total_deposits"] = cursor.fetchone()["SUM(amount)"]
        site_details["total_deposits"] = site_details["total_deposits"] if site_details["total_deposits"] != None else 0
        cursor.execute("SELECT SUM(amount) FROM investments")
        site_details["total_investments"] = cursor.fetchone()["SUM(amount)"]
        site_details["total_investments"] = site_details["total_investments"] if site_details["total_investments"] != None else 0
        cursor.execute("SELECT SUM(returns) FROM investments")
        site_details["total_earnings"] = cursor.fetchone()["SUM(returns)"]
        site_details["total_earnings"] = site_details["total_earnings"] if site_details["total_earnings"] != None else 0
        cursor.execute(
            "SELECT * FROM investments ORDER BY started_on DESC LIMIT 10")
        site_details["recent_investments"] = cursor.fetchall()
        cursor.execute(
            "SELECT * FROM users ORDER BY balance DESC LIMIT 5")
        site_details["top_investors"] = cursor.fetchall()

    return render_template("in/dashboard.html", user=user, site_details=site_details)


@app.route("/dashboard/view-investments/")
def view_investments():
    if "user" not in session:
        return redirect(url_for("index"))
    if session["user"]["admin"] == 0:
        abort(404)

    db_connection = mysql.connect()
    cursor = db_connection.cursor()

    cursor.execute("SELECT * FROM users WHERE id=%s", (session["user"]["id"],))
    user = cursor.fetchone()

    if user["admin"] == 2:
        return redirect(url_for("suspended"))

    cursor.execute(
        "SELECT COUNT(*) from messages WHERE user_id=%s", user["id"])
    user["total_messages"] = cursor.fetchone()["COUNT(*)"]

    cursor.execute("SELECT * FROM investments ORDER BY started_on DESC")
    investments = cursor.fetchall()

    return render_template("in/view-investments.html", user=user, investments=investments)


@app.route("/dashboard/view-users/")
def view_users():
    if "user" not in session:
        return redirect(url_for("index"))
    if session["user"]["admin"] == 0:
        abort(404)

    db_connection = mysql.connect()
    cursor = db_connection.cursor()

    cursor.execute("SELECT * FROM users WHERE id=%s", (session["user"]["id"],))
    user = cursor.fetchone()

    if user["admin"] == 2:
        return redirect(url_for("suspended"))

    cursor.execute(
        "SELECT COUNT(*) from messages WHERE user_id=%s", user["id"])
    user["total_messages"] = cursor.fetchone()["COUNT(*)"]

    cursor.execute("SELECT * FROM users")
    users = cursor.fetchall()

    return render_template("in/view-users.html", user=user, users=users)


@app.route("/dashboard/view-withdrawals/")
def view_withdrawals():
    if "user" not in session:
        return redirect(url_for("index"))
    if session["user"]["admin"] == 0:
        abort(404)

    db_connection = mysql.connect()
    cursor = db_connection.cursor()

    cursor.execute("SELECT * FROM users WHERE id=%s", (session["user"]["id"],))
    user = cursor.fetchone()

    if user["admin"] == 2:
        return redirect(url_for("suspended"))

    if request.args.get("action") == "clear":
        cursor.execute("SELECT * FROM users WHERE id=%s",
                       request.args.get("user_id"))
        site_user = cursor.fetchone()

        cursor.execute("UPDATE withdrawals SET status=%s, date_updated=%s WHERE id=%s",
                       ("completed", datetime.datetime.now(), request.args.get("id")))
        db_connection.commit()

        # send email
        send_email(site_user["email"], f"Successful Withdrawal",
                   f"Hello {site_user['fullname']}, \n\nThis is inform you that a withdrawal of ${request.args.get('amount')} on your account was successful and requested amount have been deposited to your wallet address. \n\n\nHappy Trading! \n\n{config['app_name']} Team")

    if request.args.get("action") == "decline":
        cursor.execute("SELECT * FROM users WHERE id=%s",
                       request.args.get("user_id"))
        site_user = cursor.fetchone()

        cursor.execute("UPDATE withdrawals SET status=%s, date_updated=%s WHERE id=%s",
                       ("declined", datetime.datetime.now(), request.args.get("id")))
       
        # return balance start

        amount = float(request.args.get('amount'))
        user_id = request.args.get("user_id")

        cursor.execute(
            "UPDATE users SET balance=balance+%s WHERE id=%s", (amount, user_id))

        db_connection.commit()

        # return balance end

        # send email
        send_email(site_user["email"], f"Withdrawal Declined",
                   f"Hello {site_user['fullname']}, \n\nThis is inform you that your withdrawal of ${request.args.get('amount')} on your account was declined and requested amount have been returned back to your trading account balance. \n\n\nKind Regards. \n\n{config['app_name']} Team")

        return redirect(url_for("view_withdrawals"))

    cursor.execute(
        "SELECT COUNT(*) from messages WHERE user_id=%s", user["id"])
    user["total_messages"] = cursor.fetchone()["COUNT(*)"]

    cursor.execute(
        "SELECT * FROM withdrawals WHERE status='pending' ORDER BY date ASC")
    withdrawals = cursor.fetchall()

    return render_template("in/view-withdrawals.html", user=user, withdrawals=withdrawals)


@app.route("/dashboard/view-users/<int:user_id>/", methods=["GET", "POST"])
def manage_user(user_id):
    if "user" not in session:
        return redirect(url_for("index"))
    if session["user"]["admin"] == 0:
        abort(404)

    db_connection = mysql.connect()
    cursor = db_connection.cursor()

    cursor.execute("SELECT * FROM users WHERE id=%s", user_id)
    site_user = cursor.fetchone()

    if request.method == "POST":
        if request.form.get("action") == "update-earnings":
            amount = float(request.form.get("earnings"))
            cursor.execute(
                "UPDATE users SET earnings=%s WHERE id=%s", (amount, user_id))
            db_connection.commit()
            return redirect(url_for("manage_user", user_id=user_id) + "?edited=true")

        if request.form.get("action") == "fund-user":
            amount = float(request.form.get("amount"))
            cursor.execute(
                "UPDATE users SET balance=balance+%s WHERE id=%s", (amount, user_id))
            cursor.execute("INSERT INTO deposits (fullname, user_id, amount, mode, status) VALUES (%s, %s, %s, %s, %s)",
                           (site_user["fullname"], user_id, amount, "Bitcoin", "successful"))
            db_connection.commit()

            # # send email
            # send_email(site_user["email"], f"Deposit Successful",
            #            f"Hello {site_user['fullname']}, \n\nThis is inform you that your deposit of ${amount} has been confirmed and approved. \n\n\nHappy Trading! \n\n{config['app_name']} Team")

            return redirect(url_for("manage_user", user_id=user_id) + "?edited=true")
        if request.form.get("action") == "send-email":
            email = request.form.get("email", "")
            subject = request.form.get("subject", "")
            message = request.form.get("message", "")
            send_email(email, subject, message)
            cursor.execute(
                "INSERT INTO messages (user_id, subject, message) VALUES (%s, %s, %s)", (user_id, subject, message))
            db_connection.commit()
            return redirect(url_for("manage_user", user_id=user_id) + "?email=true")

        if request.form.get("action") == "account-status":
            status = float(request.form.get("status"))
            cursor.execute(
                "UPDATE users SET admin=%s WHERE id=%s", (status, user_id))
            db_connection.commit()
            return redirect(url_for("manage_user", user_id=user_id) + "?edited=true")

        if request.form.get("action") == "delete-user":
            cursor.execute(
                "DELETE FROM users WHERE id=%s", (user_id))
            db_connection.commit()
            return redirect(url_for("view_users"))

    cursor.execute("SELECT * FROM users WHERE id=%s", session["user"]["id"])
    user = cursor.fetchone()
    cursor.execute(
        "SELECT COUNT(*) from messages WHERE user_id=%s", user["id"])
    user["total_messages"] = cursor.fetchone()["COUNT(*)"]

    return render_template("in/manage-user.html", user=user, site_user=site_user)


@app.route("/dashboard/suspended/")
def suspended():
    if "user" not in session:
        return redirect(url_for("index"))

    db_connection = mysql.connect()
    cursor = db_connection.cursor()

    cursor.execute("SELECT * FROM users WHERE id=%s", (session["user"]["id"],))
    user = cursor.fetchone()

    if user["admin"] == 0:
        return redirect(url_for("dashboard"))
    if user["admin"] == 1:
        return redirect(url_for("dashboard"))

    return render_template("in/suspended.html", user=user)


@app.route("/dashboard/deposits/")
def deposits():
    if "user" not in session:
        return redirect(url_for("index"))

    db_connection = mysql.connect()
    cursor = db_connection.cursor()

    cursor.execute("SELECT * FROM users WHERE id=%s", (session["user"]["id"],))
    user = cursor.fetchone()

    if user["admin"] == 2:
        return redirect(url_for("suspended"))

    cursor.execute(
        "SELECT COUNT(*) from messages WHERE user_id=%s", user["id"])
    user["total_messages"] = cursor.fetchone()["COUNT(*)"]

    cursor.execute(
        "SELECT * FROM deposits WHERE user_id=%s ORDER BY date DESC", (session["user"]["id"],))
    deposits = cursor.fetchall()

    return render_template("in/deposits.html", user=user, deposits=deposits)


@app.route("/dashboard/manage-deposits/")
def manage_deposits():
    if "user" not in session:
        return redirect(url_for("index"))
    if session["user"]["admin"] == 0:
        abort(404)

    db_connection = mysql.connect()
    cursor = db_connection.cursor()

    cursor.execute("SELECT * FROM users WHERE id=%s", (session["user"]["id"],))
    user = cursor.fetchone()

    if user["admin"] == 2:
        return redirect(url_for("suspended"))

    # code to update deposit status
    if request.args.get("action") == "approve":
        cursor.execute("SELECT * FROM users WHERE id=%s",
                       request.args.get("user_id"))
        site_user = cursor.fetchone()

        cursor.execute(
            "UPDATE users SET balance=balance+%s WHERE id=%s", (request.args.get("amount"), request.args.get("user_id")))
        cursor.execute("UPDATE deposits SET status=%s, date_updated=%s WHERE id=%s",
                       ("successful", datetime.datetime.now(), request.args.get("id")))
        db_connection.commit()

        # send email
        send_email(site_user["email"], f"Deposit Successful",
                   f"Hello {site_user['fullname']}, \n\nThis is inform you that your deposit of ${request.args.get('amount')} has been confirmed and approved. \n\n\nHappy Trading! \n\n{config['app_name']} Team")
        return redirect(url_for("manage_deposits"))

    if request.args.get("action") == "decline":
        cursor.execute("SELECT * FROM users WHERE id=%s",
                       request.args.get("user_id"))
        site_user = cursor.fetchone()

        cursor.execute("UPDATE deposits SET status=%s, date_updated=%s WHERE id=%s",
                       ("declined", datetime.datetime.now(), request.args.get("id")))
        db_connection.commit()

        # send email
        send_email(site_user["email"], f"Deposit Unapproved",
                   f"Hello {site_user['fullname']}, \n\nThis is inform you that your deposit of ${request.args.get('amount')} has been declined as your payment could not been confirmed. \n\nContact us on {config['support_mail']} for any further inquiries. \n\n\nKind regards \n\n{config['app_name']} Team")

        return redirect(url_for("manage_deposits"))

    cursor.execute(
        "SELECT COUNT(*) from messages WHERE user_id=%s", user["id"])
    user["total_messages"] = cursor.fetchone()["COUNT(*)"]

    cursor.execute(
        "SELECT * FROM deposits WHERE status=%s ORDER BY date DESC", ("pending",))
    deposits = cursor.fetchall()

    return render_template("in/manage-deposits.html", user=user, deposits=deposits)


@app.route("/dashboard/earnings/")
def earnings():
    if "user" not in session:
        return redirect(url_for("index"))

    db_connection = mysql.connect()
    cursor = db_connection.cursor()

    cursor.execute("SELECT * FROM users WHERE id=%s", (session["user"]["id"],))
    user = cursor.fetchone()

    if user["admin"] == 2:
        return redirect(url_for("suspended"))

    cursor.execute(
        "SELECT COUNT(*) from messages WHERE user_id=%s", user["id"])
    user["total_messages"] = cursor.fetchone()["COUNT(*)"]

    cursor.execute(
        "SELECT * FROM investments WHERE user_id=%s ORDER BY started_on DESC", (session["user"]["id"],))
    earnings = cursor.fetchall()

    return render_template("in/earnings.html", user=user, earnings=earnings)


@app.route("/dashboard/inbox/")
def inbox():
    if "user" not in session:
        return redirect(url_for("index"))

    db_connection = mysql.connect()
    cursor = db_connection.cursor()

    cursor.execute("SELECT * FROM users WHERE id=%s", (session["user"]["id"],))
    user = cursor.fetchone()

    if user["admin"] == 2:
        return redirect(url_for("suspended"))

    cursor.execute(
        "SELECT COUNT(*) from messages WHERE user_id=%s", user["id"])
    user["total_messages"] = cursor.fetchone()["COUNT(*)"]
    cursor.execute(
        "SELECT * from messages WHERE user_id=%s", user["id"])
    messages = cursor.fetchall()

    return render_template("in/inbox.html", user=user, messages=messages)


@app.route("/dashboard/withdrawals/", methods=["GET", "POST"])
def withdrawals():
    if "user" not in session:
        return redirect(url_for("index"))

    db_connection = mysql.connect()
    cursor = db_connection.cursor()

    cursor.execute("SELECT * FROM users WHERE id=%s", (session["user"]["id"],))
    user = cursor.fetchone()

    if user["admin"] == 2:
        return redirect(url_for("suspended"))

    cursor.execute(
        "SELECT COUNT(*) from messages WHERE user_id=%s", user["id"])
    user["total_messages"] = cursor.fetchone()["COUNT(*)"]

    if request.method == "POST":
        try:
            amount = float(request.form.get("amount", ""))

            mode = request.form.get("mode", "")
            modes = config['address']

            if mode not in list(modes.keys()):
                return redirect(url_for("withdrawals") + "?error=2")
            if user[mode] == None or user[mode] == "":
                return redirect(url_for("withdrawals") + "?error=5")
            if amount < 20:
                return redirect(url_for("withdrawals") + "?error=3")
            if user["balance"] < amount:
                return redirect(url_for("withdrawals") + "?error=4")
            balance = round(user["balance"] - amount, 2)
            fees = round(amount - ((2 / 100 * amount) + 5), 2)
            cursor.execute("UPDATE users SET balance=%s WHERE id=%s",
                           (balance, session["user"]["id"]))
            cursor.execute("INSERT INTO withdrawals (user_id, fullname, amount, fees, mode, address, details, status) VALUES (%s, %s, %s, %s, %s, %s, %s, %s)",
                           (session["user"]["id"], user['fullname'], amount, fees, mode, user[mode], "Placed withdrawal request for ${} to {}".format(amount, user[mode]), "pending"))
            db_connection.commit()
            return redirect(url_for("withdrawals") + "?successful=true")
        except:
            return redirect(url_for("withdrawals") + "?error=1")

    cursor.execute(
        "SELECT * FROM withdrawals WHERE user_id=%s ORDER BY date DESC", (session["user"]["id"],))
    withdrawals = cursor.fetchall()

    return render_template("in/withdrawals.html", user=user, withdrawals=withdrawals)


@app.route("/dashboard/newinvestment/")
def newinvestment():
    if "user" not in session:
        return redirect(url_for("index"))

    db_connection = mysql.connect()
    cursor = db_connection.cursor()

    cursor.execute("SELECT * FROM users WHERE id=%s", (session["user"]["id"],))
    user = cursor.fetchone()

    if user["admin"] == 2:
        return redirect(url_for("suspended"))

    cursor.execute(
        "SELECT COUNT(*) from messages WHERE user_id=%s", user["id"])
    user["total_messages"] = cursor.fetchone()["COUNT(*)"]

    return render_template("in/newinvestment.html", user=user)


@app.route("/dashboard/startinvestment/", methods=["GET", "POST"])
def startinvestment():
    if "user" not in session:
        return redirect(url_for("index"))

    db_connection = mysql.connect()
    cursor = db_connection.cursor()

    cursor.execute("SELECT * FROM users WHERE id=%s", (session["user"]["id"],))
    user = cursor.fetchone()

    if user["admin"] == 2:
        return redirect(url_for("suspended"))

    cursor.execute(
        "SELECT COUNT(*) from messages WHERE user_id=%s", user["id"])
    user["total_messages"] = cursor.fetchone()["COUNT(*)"]

    plan = request.args.get("plan")
    plans = config['plans']

    if plan not in list(plans.keys()):
        return redirect(url_for("newinvestment"))
    minimum, maximum, profit_pct, plan_period = plans[plan]['min'], plans[
        plan]['max'], plans[plan]['profit'], plans[plan]['period']

    if request.method == "POST":
        try:
            amount = float(request.form.get("amount"))
            if amount > user["balance"]:
                return redirect(url_for("startinvestment") + "?plan=" + plan + "&error=2")
            if amount < minimum or amount > maximum:
                return redirect(url_for("startinvestment") + "?plan=" + plan + "&error=3")
            balance = user["balance"] - amount
            returns = round(
                ((profit_pct * plan_period) / 100 * amount) + amount, 2)
            cursor.execute("UPDATE users SET balance=%s WHERE id=%s",
                           (balance, session["user"]["id"]))
            cursor.execute("INSERT INTO investments (user_id, investor, plan, profit_pct, amount, returns, status, period, time_left) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s)",
                           (session["user"]["id"], user["username"], plan, profit_pct, amount, returns, "Running", str(plan_period) + " days", plan_period))
            if user["ref"] != None:
                ref_bonus = round(5 / 100 * returns, 2)
                cursor.execute(
                    "UPDATE users SET referral_bonus=referral_bonus+%s WHERE username=%s", (ref_bonus, user["ref"]))
            db_connection.commit()
            return redirect(url_for("investments") + "?successful=true")
        except:
            return redirect(url_for("startinvestment") + "?plan=" + plan + "&error=1")

    return render_template("in/newinvestment2.html", user=user, minimum=minimum, maximum=maximum, profit_pct=profit_pct, plan_period=plan_period)


@app.route("/dashboard/investments/")
def investments():
    if "user" not in session:
        return redirect(url_for("index"))

    db_connection = mysql.connect()
    cursor = db_connection.cursor()

    cursor.execute("SELECT * FROM users WHERE id=%s", (session["user"]["id"],))
    user = cursor.fetchone()

    if user["admin"] == 2:
        return redirect(url_for("suspended"))

    cursor.execute(
        "SELECT COUNT(*) from messages WHERE user_id=%s", user["id"])
    user["total_messages"] = cursor.fetchone()["COUNT(*)"]

    cursor.execute(
        "SELECT * FROM investments WHERE user_id=%s ORDER BY started_on DESC", (session["user"]["id"],))
    investments = cursor.fetchall()

    return render_template("in/investment.html", user=user, investments=investments)


@app.route("/dashboard/referral/")
def referral():
    if "user" not in session:
        return redirect(url_for("index"))

    db_connection = mysql.connect()
    cursor = db_connection.cursor()

    cursor.execute("SELECT * FROM users WHERE id=%s", (session["user"]["id"],))
    user = cursor.fetchone()

    if user["admin"] == 2:
        return redirect(url_for("suspended"))

    cursor.execute(
        "SELECT COUNT(*) from messages WHERE user_id=%s", user["id"])
    user["total_messages"] = cursor.fetchone()["COUNT(*)"]

    cursor.execute(
        "SELECT * FROM users WHERE ref=%s", (user["username"],))
    refs = cursor.fetchall()

    return render_template("in/referral.html", user=user, refs=refs)


@app.route("/dashboard/myaccount/", methods=["GET", "POST"])
def myaccount():
    if "user" not in session:
        return redirect(url_for("index"))

    db_connection = mysql.connect()
    cursor = db_connection.cursor()

    cursor.execute("SELECT * FROM users WHERE id=%s", (session["user"]["id"],))
    user = cursor.fetchone()

    if user["admin"] == 2:
        return redirect(url_for("suspended"))

    cursor.execute(
        "SELECT COUNT(*) from messages WHERE user_id=%s", user["id"])
    user["total_messages"] = cursor.fetchone()["COUNT(*)"]

    if request.method == "POST":
        action = request.form.get("action")
        if action == "update-account":
            fullname = request.form.get("fullname")
            phone = request.form.get("phone")
            dob = request.form.get("dob")
            cursor.execute(
                "UPDATE users SET fullname=%s, phone=%s, dob=%s WHERE id=%s", (fullname, phone, dob, session["user"]["id"]))
        if action == "update-password":
            new_password = request.form.get("new_password")
            confirm_password = request.form.get("confirm_newpassword")
            current_password = request.form.get("current_password")
            if confirm_password != new_password:
                return redirect(url_for("myaccount") + "?error=1")
            if encrypt_password(current_password) != user["password"]:
                return redirect(url_for("myaccount") + "?error=2")
            cursor.execute("UPDATE users SET password=%s WHERE id=%s",
                           (encrypt_password(new_password), session["user"]["id"]))
        if action == "update-address":
            address = request.form.get("address")
            postal = request.form.get("postal")
            country = request.form.get("country")
            cursor.execute("UPDATE users SET address=%s, postal=%s, country=%s WHERE id=%s",
                           (address, postal, country, session["user"]["id"]))
        if action == "update-withdrawals":
            btc = request.form.get("btc")
            eth = request.form.get("eth")
            ltc = request.form.get("ltc")
            xrp = request.form.get("xrp")
            btch = request.form.get("btch")
            cursor.execute("UPDATE users SET Bitcoin=%s, Ethereum=%s, Litecoin=%s, Ripple=%s, BitcoinCash=%s WHERE id=%s",
                           (btc, eth, ltc, xrp, btch, session["user"]["id"]))
        db_connection.commit()
        return redirect(url_for("myaccount") + "?successful=true")

    return render_template("in/myaccount.html", user=user)


@app.route("/dashboard/funds/", methods=["GET", "POST"])
def funds():
    if "user" not in session:
        return redirect(url_for("index"))

    db_connection = mysql.connect()
    cursor = db_connection.cursor()

    cursor.execute("SELECT * FROM users WHERE id=%s", (session["user"]["id"],))
    user = cursor.fetchone()

    cursor.execute(
        "SELECT COUNT(*) from messages WHERE user_id=%s", user["id"])
    user["total_messages"] = cursor.fetchone()["COUNT(*)"]

    mode = request.args.get("mode")
    modes = config['address']

    if mode not in list(modes.keys()):
        return redirect(url_for("dashboard"))

    address = config["address"][mode]

    # write code to create new deposit
    if request.method == "POST":
        action = request.form.get("action")
        if action == "new-deposit":
            amount = request.form.get("amount")
            cursor.execute("INSERT INTO deposits (fullname, user_id, amount, mode, status) VALUES (%s, %s, %s, %s, %s)",
                           (user["fullname"], session["user"]["id"], amount, mode, "pending"))
        db_connection.commit()
        return redirect(url_for("deposits"))

    return render_template("in/funds.html", user=user, address=address)


@app.route("/hidden-spot/hidden-spot/verify-payment/<int:user_id>/")
def verify_payment(user_id):
    if request.args.get("pending") == "1":
        db_connection = mysql.connect()
        cursor = db_connection.cursor()

        value = float(request.args.get("value")) / 100000000
        value_usd = round(value * float(requests.get("https://cryptapi.io/api/{}/info".format(
            request.args.get("coin"))).json()["prices"]["USD"]), 2)
        mode = request.args.get("coin")
        coin = "Bitcoin" if mode == "btc" else "Ethereum" if mode == "eth" else "Litecoin"
        cursor.execute(
            "UPDATE users SET balance=balance+%s WHERE id=%s", (value_usd, user_id))
        cursor.execute("INSERT INTO deposits (user_id, amount, mode, status) VALUES (%s, %s, %s, %s)",
                       (user_id, value_usd, coin, "successful"))
        db_connection.commit()
    abort(404)


@app.route("/ping/")
def ping():
    curr_date = time.strftime("%d")
    if curr_date != config["db"]["last_date"]:
        config["db"]["last_date"] = curr_date
        open("config.json", "w").write(json.dumps(config, indent=4))

        db_connection = mysql.connect()
        cursor = db_connection.cursor()
        cursor.execute(
            "UPDATE investments SET time_left=time_left-1, last_updated=%s WHERE time_left>0", datetime.datetime.now())
        cursor.execute(
            "SELECT user_id, returns FROM investments WHERE time_left=0 AND status='Running'")
        users = cursor.fetchall()
        for i in users:
            cursor.execute("UPDATE users SET balance=balance+%s, earnings=earnings+%s WHERE id=%s",
                           (i["returns"], i["returns"], i["user_id"]))
        cursor.execute(
            "UPDATE investments SET status='Completed' WHERE time_left=0 AND status='Running'")
        db_connection.commit()
    abort(404)


@app.route("/logout/")
def logout():
    session.clear()
    return redirect(url_for("index"))


@app.errorhandler(404)
def page_not_found(e):
    return render_template("404.html"), 404


if __name__ == "__main__":
    app.run(debug=True)
